/*
=====================================================================
	RangedPenalty.cpp (Version 1.01)
    Program to determine what RPG ranged fire penalties should be.
    Assumes inverse-square law and normal-distribution target (x,y)
    v.101: Synchronizing with archery article in Dragon #58, p. 47
	Copyright (c) 2010-2011 Daniel R. Collins. All rights reserved.
 	See the bottom of this file for any licensing information.
=====================================================================
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include "normal.h"

/*
---------------------------------------------------------------------
  Globals
---------------------------------------------------------------------
*/

int seed;
const int NUM_SHOTS = 100000;
const double START_RADIUS = 2;          // In feet; standard archery target/man
//const double START_RADIUS = 24;       // In feet; Royal Company distance compete
const double PRECISION = 6.8;           // Expert, per Barrow article
//const double PRECISION = 1.9;         // Novice [divides normal (x,y)]
const double INC_FACTOR = 1.071773463;  // 10th root of 2


/*
---------------------------------------------------------------------
  randomShot()
    Random shot at center (0,0) of target radius r.
    Deviation in each axis (x,y) is standard normal N(0,1).
    Returns true if shot hits within target area.
---------------------------------------------------------------------
*/

bool randomShot (double r) {
  double x = r8_normal_01(&seed) / PRECISION;
  double y = r8_normal_01(&seed) / PRECISION;
  return (x*x + y*y <= r*r); // Pythagoras
}


/*
---------------------------------------------------------------------
  manyShotRatio()
    Fires many shots; returns ratio that hit
---------------------------------------------------------------------
*/

double manyShotRatio (int shots, double r) {
  int hits = 0;
  for (int i = 0; i < shots; i++) {
    if (randomShot(r))
      hits++;
  }
  return (double) hits/shots;
}


/*
---------------------------------------------------------------------
  makeTable()
    Makes table of resulting data
---------------------------------------------------------------------
*/

void makeTable (void) {

  // Title
  printf("Percent to hit at different ranges\n\n");
  
  // Header
  printf("Range\tPercent\n");
  printf("-----\t-------\n");

  // Body
  double range = 10; // yards
  double radius = START_RADIUS;
  for (; range <= 240; range *= INC_FACTOR) {
    printf("%f\t", range);
    printf("%f\n", manyShotRatio(NUM_SHOTS, radius) * 100);
    radius /= INC_FACTOR;
  }

  // Tail
  printf("\n\n");
}


/*
---------------------------------------------------------------------
  main()
---------------------------------------------------------------------
*/

int main (int argc, char *argv[]) {
  seed = time(NULL);
  makeTable();
  return 0;
}


/*
==========================================================================
LICENSING INFORMATION

This program is free software; you can redistribute it and/or 
modify it under the terms of the GNU General Public License as 
published by the Free Software Foundation; either version 2 of 
the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, 
but WITHOUT ANY WARRANTY; without even the implied warranty of 
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the 
GNU General Public License for more details.

You should have received a copy of the GNU General Public License 
along with this program; if not, write to the Free Software Foundation, 
Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.

The author may be contacted by email at: delta@superdan.net
==========================================================================
*/

